// Server.java

import java.util.*;

class Account extends org.omg.CORBA.DynamicImplementation {
  Account(float balance) {
    super(null, "IDL:Bank/Account:1.0");
    _balance = balance;
  }
  public void invoke(org.omg.CORBA.ServerRequest request) {
    // make the operation name is correct
    if(!request.op_name().equals("balance")) {
      throw new org.omg.CORBA.BAD_OPERATION();
    }
    // no parameters, so simply invoke the balance operation
    float balance = this.balance();
    // create an any for the result
    org.omg.CORBA.Any balanceAny = _orb().create_any();
    // put in the balance, 
    balanceAny.insert_float(balance);
    // set the request's result
    request.result(balanceAny);
  }
  float balance() {
    return _balance;
  }
  private float _balance;
}

class AccountManager extends org.omg.CORBA.DynamicImplementation {
  AccountManager(String name) {
    super(name, "IDL:Bank/AccountManager:1.0");
  }
  public void invoke(org.omg.CORBA.ServerRequest request) {
    // make the operation name is correct
    if(!request.op_name().equals("open")) {
      throw new org.omg.CORBA.BAD_OPERATION();
    }
    // create an empty parameter list
    org.omg.CORBA.NVList params = _orb().create_list(0);
    // create an any for the name parameter
    org.omg.CORBA.Any nameAny = _orb().create_any();
    // set the any's type to be a string
    nameAny.type(_orb().get_primitive_tc(
		org.omg.CORBA.TCKind.tk_string));
    // add "in" the parameter to the parameter list
    params.add_value("name", nameAny, org.omg.CORBA.ARG_IN.value);
    // obtains the params from the request
    request.params(params);
    // get the name parameter from the any
    String name = nameAny.extract_string();
    // invoke the open operation
    Account account = this.open(name);
    // create an any for the result
    org.omg.CORBA.Any accountAny = _orb().create_any();
    // put in the new account object
    accountAny.insert_Object(account);
    // set the request's result
    request.result(accountAny);
  }
  public synchronized Account open(String name) {
    // Lookup the account in the account dictionary.
    Account account = (Account) _accounts.get(name);
    if(account == null) {
      // Create a new account with between 0 and 1000 dollars.
      float balance = Math.abs(_random.nextInt()) % 100000 / 100f;
      account = new Account(balance);
      // Export the new object reference.
      _boa().obj_is_ready(account);
      System.out.println("Created " + name + "'s account: " + account);
      // Save the account in the account dictionary.
      _accounts.put(name, account);
    }
    // Return the account.
    return account;
  }
  private Dictionary _accounts = new Hashtable();
  private Random _random = new Random();
}

public class Server {
  public static void main(String[] args) {
    // Initialize the ORB.
    org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,null);
    // Initialize the BOA.
    org.omg.CORBA.BOA boa = orb.BOA_init();
    // Create the account manager object.
    AccountManager manager = 
      new AccountManager("BankManager");
    // Export the newly create object.
    boa.obj_is_ready(manager);
    System.out.println(manager + " is ready.");
    // Wait for incoming requests
    boa.impl_is_ready();
  }

}

