// Server.java

import java.util.*;
import com.visigenic.vbroker.interceptor.*;

class AccountImpl extends Bank._AccountImplBase {
  AccountImpl(float balance) {
    _balance = balance;
  }
  public float balance() {
    return _balance;
  }
  private float _balance;
}

class AccountManagerImpl extends Bank._AccountManagerImplBase {
  static int _counter;
  AccountManagerImpl(String name) {
    super(name);
  }
  public synchronized Bank.Account open(String name) {
    // Lookup the account in the account dictionary.
    Bank.Account account = (Bank.Account) _accounts.get(name);
    if(account == null) {
      // Create a new account with between 0 and 1000 dollars.
      float balance = Math.abs(_random.nextInt()) % 100000 / 100f;
      account = new AccountImpl(balance);
      System.out.println("Created " + name + "'s account: " + account);
      // Export the new object reference.
      _boa().obj_is_ready(account);
      // Save the account in the account dictionary.
      _accounts.put(name, account);
    }
    // Return the account.
    return account;
  }
  private Dictionary _accounts = new Hashtable();
  private Random _random = new Random();
}

class MyImplEventHandler implements ImplEventHandler {

  private String _name;

  MyImplEventHandler(String name) {
    _name = name;
  }

  public void bind(ConnectionInfo info, org.omg.CORBA.Principal princ, org.omg.CORBA.Object obj) {
    System.out.println(_name + ": bind: " +
		       "\n\tinfo=" + toString(info) + 
		       "\n\tprincipal=" + princ +
		       "\n\tobject=" + obj._repository_id());
  }

  public void unbind(ConnectionInfo info, org.omg.CORBA.Principal princ, org.omg.CORBA.Object obj) {
    System.out.println(_name + ": unbind: " +
		       "\n\tinfo=" + toString(info) + 
		       "\n\tprincipal=" + princ +
		       "\n\tobject=" + obj._repository_id());
  }

  public void client_aborted(ConnectionInfo info, org.omg.CORBA.Object obj) {
    System.out.println(_name + ": client_aborted: " +
		       "\n\tinfo=" + toString(info) + 
		       "\n\tobject=" + obj._repository_id());
  }

  public void pre_method(ConnectionInfo info, org.omg.CORBA.Principal princ, String  operation_name, org.omg.CORBA.Object obj) { 
    System.out.println(_name + ": pre_method: " +
		       "\n\tinfo=" + toString(info) + 
		       "\n\tprincipal=" + princ +
		       "\n\toperation=" + operation_name + 
		       "\n\tobject=" + obj._repository_id());
  }

  public void post_method(ConnectionInfo info, org.omg.CORBA.Principal princ, String  operation_name, org.omg.CORBA.Object obj) {
    System.out.println(_name + ": post_method: " +
		       "\n\tinfo=" + toString(info) + 
		       "\n\tprincipal=" + princ +
		       "\n\toperation=" + operation_name + 
		       "\n\tobject=" + obj._repository_id());
  }

  public void post_method_exception(ConnectionInfo info, org.omg.CORBA.Principal princ, String  operation_name, String exception_rep_id, org.omg.CORBA.Object obj) {
    System.out.println(_name + ": post_method_exception: " +
		       "\n\tinfo=" + toString(info) + 
		       "\n\tprincipal=" + princ +
		       "\n\toperation=" + operation_name + 
		       "\n\texception=" + exception_rep_id + 
		       "\n\tobject=" + obj._repository_id());
  }

  private String toString(ConnectionInfo info) {
    return "ConnectionInfo[hostname=" + info.hostname +
      ",port=" + info.port + "]";
  }

}

public class Server {
  public static void main(String[] args) {
    try {
      // Initialize the ORB.
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,null);
      // Initialize the BOA.
      org.omg.CORBA.BOA boa = orb.BOA_init();
      
      HandlerRegistry registry = HandlerRegistryHelper.narrow(
	  orb.resolve_initial_references("HandlerRegistry"));

      try {
        registry.reg_glob_impl_handler
	   (new MyImplEventHandler("global"));
      }
      catch (HandlerExists e) {
	System.err.println(e);
      }

      // Create the account manager object.
      AccountManagerImpl manager = 
	new AccountManagerImpl("BankManager");
      
      try {
        registry.reg_obj_impl_handler
	  (manager, new MyImplEventHandler("object"));
      } 
      catch (HandlerExists e) {
	System.err.println(e);
      }
      catch (InvalidObject e) {
	System.err.println(e);
      }

      // Export the newly create object.
      boa.obj_is_ready(manager);
      System.out.println(manager + " is ready.");
      // Wait for incoming requests
      boa.impl_is_ready();
    }
    catch(Exception e) {
      System.err.println(e);
    }
  }
}

