// Console.java

import java.util.*; 
import java.io.*;

import com.visigenic.vbroker.services.ORBManager.*;
import com.visigenic.vbroker.services.ORBManager.AttributeSetPackage.*;

public class Console {
  
  private static org.omg.CORBA.ORB _orb;
  private static Server _server;
  private static Adapter _adapter;

  public static void main(String[] args) {
    _orb = org.omg.CORBA.ORB.init(args,null);
    DataInputStream in = new DataInputStream(System.in);
    while(true) {
      try {
	System.out.print("ORBManager-> ");
	System.out.flush();
	if (System.getProperty("VM_THREAD_BUG") != null) {
	  while(in.available() == 0) {
	    try {
	      Thread.currentThread().sleep(100);
	    }
	    catch(InterruptedException e) {
	    }
	  }
	}
	
	String line = in.readLine();
	if(line.startsWith("c ")) {
	  try {
	    String id = line.substring("c ".length(), line.indexOf(';'));
	    String name =  line.substring(line.indexOf(';')+1, line.length());
	    org.omg.CORBA.Object dummyServer = _orb.bind( id, name, null, null);
	    org.omg.CORBA.Object serobj = dummyServer._resolve_reference("ORBManager");
	    _server = ServerHelper.narrow(serobj);
	    _adapter = null;
	    System.out.println("Connected: " + _server._repository_id());
	    continue;
	  }
	  catch (Exception e) {
	    System.out.println("Error: Invalid arguments: " + line);
	  }
	}
	else if(line.startsWith("g ")) {
	  String att = line.substring("g ".length(), line.length());
	  getAttribute( _server, att);	
	  continue;
	}
	else if(line.startsWith("s ")) {
	  try {
	    String att = line.substring("s ".length(), line.indexOf('='));
	    String value =  line.substring(line.indexOf('=')+1, line.length());
	    setAttribute( _server, att, value);	  
	    continue;
	  }
	  catch (Exception e) {
	    System.out.println("Error: Invalid arguments");
	  }
	}
	else if(line.startsWith("l")) {
	  listAllAttributes( _server);	  
	  continue;
	}
        else if(line.startsWith("C ")) {
	  String adapter = line.substring("C ".length(), line.length());
	  _adapter = _server.get_adapter(adapter);
	  System.out.println("Connected: " + _adapter._repository_id());
	  continue;
	}
	else if(line.startsWith("d")) {
	  if (_server == null) {
	    System.out.println("Error: You must be connected to a server.");
	    continue;
	  }
	  _server.shutdown();	  
	  continue;
	}
	else if(line.startsWith("G ")) {
	  String att = line.substring("G ".length(), line.length());
	  getAttribute( _adapter, att);	  
	  continue;
	}
	else if(line.startsWith("S ")) {
	  try {
	    String att = line.substring("S ".length(), line.indexOf('='));
	    String value =  line.substring(line.indexOf('=')+1, line.length());
	    setAttribute( _adapter, att, value);	  
	    continue;
	  }
	  catch (Exception e) {
	    System.out.println("Error: Invalid arguments");
	  }
	}
	else if(line.startsWith("L")) {
	  listAllAttributes( _adapter);	  
	  continue;
	}
	else if(line.startsWith("O")) {
	  listAllPersistentObjects();	  
	  continue;
	}
	else if(line.startsWith("q")) {
	  System.out.println("Quitting...");
	  break;
	}
	else {
	  System.out.println("Commands:\n" +
			     "  c <rep id>;<name> [c]connect to a server\n" +
			     "  g <attr>          [g]et an attribute of the server\n" +
			     "  s <attr>=<value>  [s]et an attribute of the server\n" +
			     "  l                 [l]ist all attributes of the server\n" +
			     "  d                 shut[d]own the server \n" +
			     "  C <kind>          [C]onnect to an adapter\n" +
			     "  G <attr>          [G]et an attribute of the adapter\n" +
			     "  S <attr>=<value>  [S]et an attribute of the adapter\n" +
			     "  L                 [L]ist all attributes of the adapter\n" +
			     "  O                 List all persistent [O]bjects\n" +
			     "  q                 [q]uit\n");
	  continue;
	}
      }
      catch(org.omg.CORBA.SystemException e) {
	e.printStackTrace();
      }
      catch(Exception e) {
	e.printStackTrace();
      }
    }
  }

  static void getAttribute(AttributeSet server, String attribute) {
    if (server == null) {
      System.out.println("Error: You must be connected to a server.");
      return;
    }
    try {
      org.omg.CORBA.Any value = server.get_attribute(attribute);
      System.out.println(attribute + ": " + value);
    }
    catch (InvalidAttributeId e) {
      System.out.println("Error: Attribute " + attribute + " does not exist.");
    } 
  }

  static void setAttribute(AttributeSet server, String attribute, String value) {
    if (server == null) {
      System.out.println("Error: You must be connected to that interface.");
      return;
    }
    try {
      org.omg.CORBA.Any valueHolder = _orb.create_any();
      valueHolder.insert_string(value);
      server.set_attribute(attribute, valueHolder);  
      System.out.println("Changed " + attribute + " to " + value);
    }
    catch (AttributeReadOnly e) {
      System.out.println("Error: "+ attribute + " is readonly");
    }
    catch (InvalidAttributeId e) {
      System.out.println("Error: "+ attribute + " does not exist");
    }
    catch (java.lang.Exception e) {
      System.out.println("Error: " + e);
    }
  }
	
  static void listAllAttributes(AttributeSet server) {
    if (server == null) {
      System.out.println("Error: You must be connected to that interface.");
      return;
    }
    Attribute[] attributes = server.get_all_attributes();
    for (int i=0; i < attributes.length; i++) {
      System.out.println("Name: " + attributes[i].id);
      System.out.println( "\tvalue: " + attributes[i].value);
      System.out.println("\treadonly: " + attributes[i].is_readonly);
    }
  }

  static void listAllPersistentObjects() {
    if (_adapter == null) {
      System.out.println("Error: You must be connected to an adapter.");
      return;
    }
    org.omg.CORBA.Object[] objects = _adapter.persistent_objects();
    for (int i=0; i < objects.length; i++) {
      System.out.println("Name: " + objects[i]._object_name() + "\n" +
			 "  ID: " + objects[i]._repository_id());
    }
  }
}

